==> db4o-5.0-mono.tar.gz: db4o-5.0/src/core/src/core/com/db4o/QResultClient.cs <==
/* Copyright (C) 2004 - 2005  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace com.db4o
{
	/// <exclude></exclude>
	internal class QResultClient : com.db4o.QueryResultImpl
	{
		private object[] i_prefetched = new object[com.db4o.YapConst.PREFETCH_OBJECT_COUNT
			];

		private int i_remainingObjects;

		private int i_prefetchCount = com.db4o.YapConst.PREFETCH_OBJECT_COUNT;

		private int i_prefetchRight;

		internal QResultClient(com.db4o.Transaction a_ta) : base(a_ta)
		{
		}

		internal QResultClient(com.db4o.Transaction a_ta, int initialSize) : base(a_ta, initialSize
			)
		{
		}

		public override bool hasNext()
		{
			lock (streamLock())
			{
				if (i_remainingObjects > 0)
				{
					return true;
				}
				return base.hasNext();
			}
		}

		public override object next()
		{
			lock (streamLock())
			{
				com.db4o.YapClient stream = (com.db4o.YapClient)i_trans.i_stream;
				stream.checkClosed();
				if (i_remainingObjects < 1)
				{
					if (base.hasNext())
					{
						i_remainingObjects = (stream).prefetchObjects(this, i_prefetched, i_prefetchCount
							);
						i_prefetchRight = i_remainingObjects;
					}
				}
				i_remainingObjects--;
				if (i_remainingObjects < 0)
				{
					return null;
				}
				if (i_prefetched[i_prefetchRight - i_remainingObjects - 1] == null)
				{
					return next();
				}
				return activate(i_prefetched[i_prefetchRight - i_remainingObjects - 1]);
			}
		}

		public override void reset()
		{
			lock (streamLock())
			{
				i_remainingObjects = 0;
				i_prefetchRight = 0;
				base.reset();
			}
		}
	}
}

==> db4o-5.5-mono.tar.gz: db4o-5.5/src/core/src/core/com/db4o/QResultClient.cs <==
/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace com.db4o
{
	/// <exclude></exclude>
	internal class QResultClient : com.db4o.QueryResultImpl
	{
		private object[] i_prefetched = new object[com.db4o.YapConst.PREFETCH_OBJECT_COUNT
			];

		private int i_remainingObjects;

		private int i_prefetchCount = com.db4o.YapConst.PREFETCH_OBJECT_COUNT;

		private int i_prefetchRight;

		internal QResultClient(com.db4o.Transaction a_ta) : base(a_ta)
		{
		}

		internal QResultClient(com.db4o.Transaction a_ta, int initialSize) : base(a_ta, initialSize
			)
		{
		}

		public override bool HasNext()
		{
			lock (StreamLock())
			{
				if (i_remainingObjects > 0)
				{
					return true;
				}
				return base.HasNext();
			}
		}

		public override object Next()
		{
			lock (StreamLock())
			{
				com.db4o.YapClient stream = (com.db4o.YapClient)i_trans.i_stream;
				stream.CheckClosed();
				if (i_remainingObjects < 1)
				{
					if (base.HasNext())
					{
						i_remainingObjects = (stream).PrefetchObjects(this, i_prefetched, i_prefetchCount
							);
						i_prefetchRight = i_remainingObjects;
					}
				}
				i_remainingObjects--;
				if (i_remainingObjects < 0)
				{
					return null;
				}
				if (i_prefetched[i_prefetchRight - i_remainingObjects - 1] == null)
				{
					return Next();
				}
				return Activate(i_prefetched[i_prefetchRight - i_remainingObjects - 1]);
			}
		}

		public override void Reset()
		{
			lock (StreamLock())
			{
				i_remainingObjects = 0;
				i_prefetchRight = 0;
				base.Reset();
			}
		}
	}
}
